/*
 * Copyright (c) Institute for System Programming of the Russian Academy of Sciences
 * All Rights Reserved
 *
 * Institute for System Programming of the Russian Academy of Sciences (ISP RAS)
 * 25 Alexander Solzhenitsyn st., Moscow, 109004, Russia
 * http://www.ispras.ru
 *
 * The present file was automatically generated on the basis of formal specifications.
 * It is distributed under the same terms and conditions as the derived specifications.
 *
 * N.B. PLEASE DO NOT MODIFY THIS FILE.
 */

package ru.ispras.microtesk.model.mips64.mmu.sim;

import java.math.BigInteger;
import ru.ispras.microtesk.model.api.Execution;
import ru.ispras.microtesk.mmu.model.api.*;
import ru.ispras.fortress.data.types.bitvector.*;

/**
 * The {@code MMU} class is automatically generated by MicroTESK.
 *
 * @author <a href="mailto:microtesk-support@ispras.ru">MicroTESK</a>
 */
public final class MMU extends Mmu<VA> {
  private static MMU instance = null;

  public static MMU get() {
    if (null == instance) {
      instance = new MMU();
    }
    return instance;
  }

  private MMU() {}

  @Override
  public int getAddressBitSize() {
    return 64;
  }

  @Override
  public int getDataBitSize() {
    return 64;
  }

  @Override
  protected VA newAddress() {
    return new VA();
  }

  @Override
  public BitVector getData(final VA va) {
    final BitVector data = BitVector.newEmpty(64);

    final PA pa = new PA();
    final BitVector isCached = BitVector.newEmpty(1);
    final BitVector l2Bypass = BitVector.newEmpty(1);
    final BitVector line = BitVector.newEmpty(256);
    final BitVector offset = BitVector.newEmpty(8);
    final BitVector bitOffset = BitVector.newEmpty(8);
    final L1.Entry l1Entry = new L1.Entry();
    final L2.Entry l2Entry = new L2.Entry();
    final MEM.Entry memEntry = new MEM.Entry();

    pa.assign(TranslateAddress.call(va));
    isCached.assign(IsCached.call(pa.cca));
    l2Bypass.assign(IsSecondaryBypass.call(pa.cca));
    offset.field(7, 6).assign(BitVector.valueOf(0x00, 2));
    offset.field(5, 0).assign(pa.value.field(5, 0));
    bitOffset.assign(BitVectorMath.shl(offset, BitVector.valueOf(0x03, 8)));
    if (isCached.equals(BitVector.valueOf(0x01, 1))) {
      if (L1.get().isHit(pa)) {
        l1Entry.assign(L1.get().getData(pa));
        line.assign(l1Entry.DATA);
      } else {
        if (l2Bypass.equals(BitVector.valueOf(0x00, 1))) {
          if (L2.get().isHit(pa)) {
            l2Entry.assign(L2.get().getData(pa));
            line.assign(l2Entry.DATA);
          } else {
            line.assign(MEM.get().getData(pa).asBitVector());
            l2Entry.TAG.assign(pa.value.field(35, 17));
            l2Entry.DATA.assign(line);
            L2.get().setData(pa).assign(l2Entry);
          }
        } else {
          line.assign(MEM.get().getData(pa).asBitVector());
        }
        l1Entry.TAG.assign(pa.value.field(35, 12));
        l1Entry.DATA.assign(line);
        L1.get().setData(pa).assign(l1Entry);
      }
    } else {
      line.assign(MEM.get().getData(pa).asBitVector());
    }
    data.assign(line.field(BitVectorMath.add(bitOffset, BitVector.valueOf(0x3F, 8)), bitOffset));

    return data;
  }

  @Override
  public BitVector setData(final VA va, final BitVector data) {
    final PA pa = new PA();
    final BitVector isCached = BitVector.newEmpty(1);
    final BitVector l2Bypass = BitVector.newEmpty(1);
    final BitVector line = BitVector.newEmpty(256);
    final BitVector offset = BitVector.newEmpty(8);
    final BitVector bitOffset = BitVector.newEmpty(8);
    final L1.Entry l1Entry = new L1.Entry();
    final L2.Entry l2Entry = new L2.Entry();
    final MEM.Entry memEntry = new MEM.Entry();

    pa.assign(TranslateAddress.call(va));
    offset.field(7, 6).assign(BitVector.valueOf(0x00, 2));
    offset.field(5, 0).assign(pa.value.field(5, 0));
    bitOffset.assign(BitVectorMath.shl(offset, BitVector.valueOf(0x03, 8)));
    isCached.assign(IsCached.call(pa.cca));
    l2Bypass.assign(IsSecondaryBypass.call(pa.cca));
    if (isCached.equals(BitVector.valueOf(0x01, 1))) {
      if (L1.get().isHit(pa)) {
        l1Entry.assign(L1.get().getData(pa));
        l1Entry.DATA.field(BitVectorMath.add(bitOffset, BitVector.valueOf(0x3F, 8)), bitOffset).assign(data);
        L1.get().setData(pa).assign(l1Entry);
      } else {
        if (l2Bypass.equals(BitVector.valueOf(0x00, 1))) {
          if (L2.get().isHit(pa)) {
            l2Entry.assign(L2.get().getData(pa));
            l2Entry.DATA.field(BitVectorMath.add(bitOffset, BitVector.valueOf(0x3F, 8)), bitOffset).assign(data);
            L2.get().setData(pa).assign(l2Entry);
          } else {
            memEntry.assign(MEM.get().getData(pa));
            memEntry.DATA.field(BitVectorMath.add(bitOffset, BitVector.valueOf(0x3F, 8)), bitOffset).assign(data);
            MEM.get().setData(pa).assign(memEntry);
            l2Entry.TAG.assign(pa.value.field(35, 17));
            l2Entry.DATA.field(BitVectorMath.add(bitOffset, BitVector.valueOf(0x3F, 8)), bitOffset).assign(data);
            L2.get().setData(pa).assign(l2Entry);
          }
        } else {
          memEntry.assign(MEM.get().getData(pa));
          memEntry.DATA.field(BitVectorMath.add(bitOffset, BitVector.valueOf(0x3F, 8)), bitOffset).assign(data);
          MEM.get().setData(pa).assign(memEntry);
        }
        l1Entry.TAG.assign(pa.value.field(35, 12));
        l1Entry.DATA.field(BitVectorMath.add(bitOffset, BitVector.valueOf(0x3F, 8)), bitOffset).assign(data);
        L1.get().setData(pa).assign(l1Entry);
      }
    } else {
      memEntry.assign(MEM.get().getData(pa));
      memEntry.DATA.field(BitVectorMath.add(bitOffset, BitVector.valueOf(0x3F, 8)), bitOffset).assign(data);
      MEM.get().setData(pa).assign(memEntry);
    }

    return null;
  }
}
